"""Test classifier_cls_by_name."""


from sklearn.svm import SVC
from sklearn.linear_model import LogisticRegression
from sklearn.naive_bayes import MultinomialNB


from skutil.estimators import classifier_cls_by_name


def test_base():
    assert classifier_cls_by_name('LogisticRegression') == LogisticRegression
    assert classifier_cls_by_name('logisticregression') == LogisticRegression
    assert classifier_cls_by_name('logreg') == LogisticRegression
    assert classifier_cls_by_name('SVC') == SVC
    assert classifier_cls_by_name('svc') == SVC
    assert classifier_cls_by_name('SVM') == SVC
    assert classifier_cls_by_name('svm') == SVC
    assert classifier_cls_by_name('MultinomialNB') == MultinomialNB
    assert classifier_cls_by_name('multinomialnb') == MultinomialNB
    assert classifier_cls_by_name('mnb') == MultinomialNB



"""Extends scikit-learn tools for hyper-parameter search."""

import copy
from collections.abc import Sequence

import numpy as np
from sklearn.model_selection import ParameterGrid


class ConstrainedParameterGrid(ParameterGrid):
    """Grid of discrete-valued parameters with constraints.

    Parameters
    ----------
    param_grid : dict of string to sequence, or sequence of such
        The parameter grid to explore, as a dictionary mapping estimator
        parameters to sequences of allowed values.

        An empty dict signifies default parameters.

        A sequence of dicts signifies a sequence of grids to search, and is
        useful to avoid exploring parameter combinations that make no sense
        or have no effect. See the examples below.
    bad_comb : list of dicts, optional
        A list of grids defining bad parameter combinations to avoid, each
        given as a dict of string to sequence. All combinations containing a
        bad combinations will be avoided.

    Example
    -------
    >>> param = {'a': [1, 2], 'b': [3, 4, 5]}
    >>> grid = ConstrainedParameterGrid(param, [{'a': [1], 'b': [4]}])
    >>> for params in grid: print(sorted(params.items()))
    [('a', 1), ('b', 3)]
    [('a', 1), ('b', 5)]
    [('a', 2), ('b', 3)]
    [('a', 2), ('b', 4)]
    [('a', 2), ('b', 5)]
    """

    def __init__(self, param_grid, bad_comb=None):
        super().__init__(param_grid)
        self.bad_comb = bad_comb
        if bad_comb is not None:
            self.bad_grids = [ParameterGrid(bad_dict) for bad_dict in bad_comb]

    def __iter__(self):
        """Iterate over the points in the grid.
        Returns
        -------
        params : iterator over dict of string to any
            Yields dictionaries mapping each estimator parameter to one of its
            allowed values.
        """
        if self.bad_comb is None:
            for params in super().__iter__():
                yield params
        else:
            bad_param_set_list = []
            for bad_param_grid in self.bad_grids:
                for bad_param_set in bad_param_grid:
                    bad_param_set_list.append(bad_param_set.items())
            for params in super().__iter__():
                bad_found = False
                for bad_param_set in bad_param_set_list:
                    if bad_param_set <= params.items():
                        bad_found = True
                        break
                if not bad_found:
                    yield params

    def partial(self, assign_grid):
        """Returns a new parameter grid by the given partial assignment.

        Parameters
        ----------
        assign_grid : dict of string to object or sequence
            A, possibly partial, assignment to the parameters of the grid. Keys
            not appearing in this grid are ignored.

        Returns
        -------
        ConstrainedParameterGrid
            A new parameter grid induced by the given partial assignment.

        Example
        -------
        >>> param = {'a': [1, 2], 'b': [3, 4, 5]}
        >>> grid = ConstrainedParameterGrid(param, [{'a': [1], 'b': [4]}])
        >>> part_grid = grid.partial({'a': 1})
        >>> for params in part_grid: print(sorted(params.items()))
        [('a', 1), ('b', 3)]
        [('a', 1), ('b', 5)]
        >>> part_grid = grid.partial({'b': [3, 4]})
        >>> for params in part_grid: print(sorted(params.items()))
        [('a', 1), ('b', 3)]
        [('a', 2), ('b', 3)]
        [('a', 2), ('b', 4)]
        """
        new_params = [copy.deepcopy(grid) for grid in self.param_grid]
        for grid in new_params:
            for key in grid:
                if key in assign_grid:
                    val = assign_grid[key]
                    if not isinstance(val, (np.ndarray, Sequence)) or (
                            isinstance(val, str)):
                        val = [val]
                    grid[key] = val
        return ConstrainedParameterGrid(
            param_grid=new_params,
            bad_comb=self.bad_comb,
        )



"""Scaler-related stuff."""

import inspect
import functools
from importlib import import_module

from decore import lazy_property


@lazy_property
def _preprocessing():
    return import_module('sklearn.preprocessing')


@functools.lru_cache(maxsize=128)
def scaler_cls_by_name(cls_name):
    """Get an sklearn scaler class by name.

    Parameters
    ----------
    cls_name : str
        The name of the sklearn scaler class to get.

    Returns
    -------
    object
        The class object of the desired scaler.

    Example
    -------
    >>> scaler_cls_by_name('RobustScaler')
    <class 'sklearn.preprocessing._data.RobustScaler'>
    >>> scaler_cls_by_name('Normalizer')
    <class 'sklearn.preprocessing._data.Normalizer'>
    """
    submodule = _preprocessing()
    return getattr(submodule, cls_name)


@functools.lru_cache(maxsize=128)
def _constructor_kwargs_by_class(klass):
    sig = inspect.signature(klass)
    return list(sig.parameters.keys())


# flake8: noqa: E501
def scaler_by_params(name, **kwargs):
    """Returns an sklearn scaler object by the given name and parameters.

    Parameters
    ----------
    name : str
        The name of the sklearn scaler class.
    **kwargs : Extra keyword arguments
        All keyword arguments supported by the consturctor of the class are
        forwared to it, while the rest are discarded.

    Returns
    -------
    object
        The class object of the desired scaler.

    Example
    -------
    >>> scaler_by_params('QuantileTransformer', n_quantiles=500)  # doctest: +SKIP
    QuantileTransformer(copy=True, ignore_implicit_zeros=False, n_quantiles=500,
              output_distribution='uniform', random_state=None,
              subsample=100000)
    """
    klass = scaler_cls_by_name(name)
    allowed_kwargs = _constructor_kwargs_by_class(klass)
    constructor_kwargs = {
        key: kwargs[key] for key in kwargs
        if key in allowed_kwargs and key != 'self'
    }
    return klass(**constructor_kwargs)



"""scikit-learn classifier wrapper calibrating after fit."""

from sklearn.base import BaseEstimator, ClassifierMixin
from sklearn.model_selection import train_test_split
# from sklearn.calibration import CalibratedClassifierCV

from .calib_clf_cv import UnsafeCalibratedClassifierCV


class CalibratingCvClassifier(BaseEstimator, ClassifierMixin):
    """A sklearn classifier wrapper using part of the train set to calibrate.

    Parameters
    ----------
    clf : classifier object implementing 'fit'
        The object to use to fit the data.
    method : str, optional
        See the same parameter for sklearn.calibration.CalibratedClassifierCV.
    val_size : integer or float, optional
        The portion of the train set to take as a validation set to calibrate
        the model with. If a float, interpreted as the ratio of the train set
        to take, and so must be between 0 and 1. If integer, interpreted as
        the number of entries to take. In both cases, the appropriate number
        of items is randomly sampled. By default, set to 0.25.
    stratify : bool, default True
        If set to True, the validation set is sampled in a stratified way.
    """
    def __init__(self, clf, method=None, val_size=None, stratify=True):
        self.clf = clf
        self.method = method
        self.val_size = val_size
        self.stratify = stratify

    def fit(self, X, y):
        """Fits the classifier

        Parameters
        ----------
        X : array-like, shape = [n_samples, n_features]
            The training input samples.
        y : array-like, shape = [n_samples]
            The target values. An array of int.

        Returns
        -------
        self : object
            Returns self.
        """
        strat = None
        if self.stratify:
            strat = y
        X_train, X_val, y_train, y_val = train_test_split(
            X, y, test_size=self.val_size, stratify=strat)
        self.clf.fit(X_train, y_train)
        self._calib = UnsafeCalibratedClassifierCV(
            base_estimator=self.clf, method=self.method, cv='prefit')
        self._calib.fit(X_val, y_val)
        return self

    def predict(self, X):
        """Predict labels.

        Parameters
        ----------
        X : array-like of shape = [n_samples, n_features]
            The input samples.

        Returns
        -------
        y : array of int of shape = [n_samples]
            Predicted labels for the given inpurt samples.
        """
        return self._calib.predict(X)

    def predict_proba(self, X):
        """Predict class probabilities for X.

        Parameters
        ----------
        X : array-like of shape = [n_samples, n_features]
            The input samples.

        Returns
        -------
        p : array of shape = [n_samples, n_classes]
            The class probabilities of the input samples. The order of the
            classes corresponds to that in the attribute classes_.
        """
        return self._calib.predict_proba(X)



"""Name to estimator class map."""

import inspect
import functools
from importlib import import_module


_ENSEMBLE = 'sklearn.ensemble'
_GAUS = 'sklearn.gaussian_process'
_LIN = 'sklearn.linear_model'
_SMV = 'sklearn.svm'
_NAIVE_BAYES = 'sklearn.naive_bayes'

_CLS_NAME_TO_PARAM_MAP = {

    # === Sklearn Submodules ===
    # Ensemble
    'AdaBoostClassifier': {
        'module': _ENSEMBLE,
        'names': ['AdaBoostClassifier', 'adaboostclassifier', 'AdaBoost',
                  'adaboost'],
    },
    'BaggingClassifier': {
        'module': _ENSEMBLE,
        'names': ['BaggingClassifier', 'baggingclassifier', 'Bagging',
                  'bagging'],
    },
    'ExtraTreesClassifier': {
        'module': _ENSEMBLE,
        'names': ['ExtraTreesClassifier', 'extratreesclassifier',
                  'ExtraTrees', 'extratrees'],
    },
    'GradientBoostingClassifier': {
        'module': _ENSEMBLE,
        'names': ['GradientBoostingClassifier', 'gradientboostingclassifier',
                  'GradientBoosting', 'gradientboosting'],
    },
    'RandomForestClassifier': {
        'module': _ENSEMBLE,
        'names': ['RandomForestClassifier', 'randomforestclassifier',
                  'RandomForest', 'randomforest'],
    },
    'VotingClassifier': {
        'module': _ENSEMBLE,
        'names': ['VotingClassifier', 'votingclassifier', 'Voting', 'voting'],
    },
    # Gaussian processes
    'GaussianProcessClassifier': {
        'module': _GAUS,
        'names': ['GaussianProcessClassifier', 'GaussianProcessClassifier',
                  'GaussianProcess', 'gaussianprocess'],
    },
    # Linear models
    'LogisticRegression': {
        'module': _LIN,
        'names': ['LogisticRegression', 'logisticregression', 'logreg', 'lr'],
    },
    'LogisticRegressionCV': {
        'module': _LIN,
        'names': ['LogisticRegressionCV', 'logisticregressioncv', 'logregcv',
                  'lrcv'],
    },
    'SGDClassifier': {
        'module': _LIN,
        'names': ['SGDClassifier', 'sgdclassifier', 'sgd'],
    },
    # SVM
    'SVC': {
        'module': _SMV,
        'names': ['SVC', 'svc', 'SVM', 'svm'],
    },
    'LinearSVC': {
        'module': _SMV,
        'names': ['LinearSVC', 'linearsvc', 'linsvm'],
    },
    # Naive Bayes
    'BernoulliNB': {
        'module': _NAIVE_BAYES,
        'names': ['BernoulliNB', 'bernoullinb', 'bnb'],
    },
    'GaussianNB': {
        'module': _NAIVE_BAYES,
        'names': ['GaussianNB', 'gaussiannb', 'gnb'],
    },
    'MultinomialNB': {
        'module': _NAIVE_BAYES,
        'names': ['MultinomialNB', 'multinomialnb', 'mnb'],
    },

    # === Others ===
    'XGBClassifier': {
        'module': 'xgboost.sklearn',
        'names': ['XGBClassifier', 'xgbclassifier', 'xgboost', 'xgb'],
    },
    'LGBMClassifier': {
        'module': 'lightgbm',
        'names': ['LGBMClassifier', 'lgbmclassifier'],
    },
    'CatBoostClassifier': {
        'module': 'catboost',
        'names': ['CatBoostClassifier', 'catboostclassifier'],
    },
}

_NAME_TO_MODULE_N_CLS_MAP = {}

for cls_name in _CLS_NAME_TO_PARAM_MAP:
    params = _CLS_NAME_TO_PARAM_MAP[cls_name]
    for name in params['names']:
        _NAME_TO_MODULE_N_CLS_MAP[name] = (params['module'], cls_name)


@functools.lru_cache(maxsize=32)
def _get_module(submodule_path):
    return import_module(submodule_path)


def classifier_cls_by_name(name):
    """Get an sklearn classifier class by name.

    Parameters
    ----------
    name : str
        The name of the sklearn classifier class to get. Can also be lower-
        cased. Also, some shorthands are supported (e.g. svm for SVC, logreg
        and lr for LogisticRegression).

    Returns
    -------
    object
        The class object of the desired classifier.

    Example
    -------
    >>> classifier_cls_by_name('LogisticRegression')
    <class 'sklearn.linear_model._logistic.LogisticRegression'>
    >>> classifier_cls_by_name('logisticregression')
    <class 'sklearn.linear_model._logistic.LogisticRegression'>
    >>> classifier_cls_by_name('logreg')
    <class 'sklearn.linear_model._logistic.LogisticRegression'>
    >>> classifier_cls_by_name('lr')
    <class 'sklearn.linear_model._logistic.LogisticRegression'>
    """
    submodule_name, cls_name = _NAME_TO_MODULE_N_CLS_MAP[name]
    submodule = _get_module(submodule_name)
    # submodule = getattr(_sklearn(), submodule_name)
    return getattr(submodule, cls_name)


@functools.lru_cache(maxsize=128)
def _constructor_kwargs_by_class(klass):
    sig = inspect.signature(klass)
    return list(sig.parameters.keys())


# flake8: noqa: E501
def classifier_by_params(name, **kwargs):
    """Returns a classifier object by the given name and parameters.

    Parameters
    ----------
    name : str
        The name of the sklearn classifier class. Can also be lower-
        cased. Also, some shorthands are supported (e.g. svm for SVC, logreg
        and lr for LogisticRegression).
    **kwargs : Extra keyword arguments
        All keyword arguments supported by the consturctor of the class are
        forwared to it, while the rest are discarded.

    Returns
    -------
    object
        The class object of the desired classifier.

    Example
    -------
    >>> classifier_by_params('LogisticRegression', penalty='l1', ignore='a') # doctest: +SKIP
    LogisticRegression(C=1.0, class_weight=None, dual=False, fit_intercept=True,
                       intercept_scaling=1, l1max_iter=100, multi_class='ovr', n_jobs=1,
              penalty='l1', random_state=None, solver='liblinear', tol=0.0001,
              verbose=0, warm_start=False)
    """
    klass = classifier_cls_by_name(name)
    allowed_kwargs = _constructor_kwargs_by_class(klass)
    model_kwargs = {
        key: kwargs[key] for key in kwargs
        if key in allowed_kwargs and key != 'self'
    }
    return klass(**model_kwargs)



"""A percentile selector you can fit once and use for many percentiles."""

from warnings import warn

import numpy as np
from sklearn.utils import safe_mask
from sklearn.utils.validation import (
    check_is_fitted,
    check_array,
)
from sklearn.feature_selection import (
    SelectPercentile,
)
from sklearn.feature_selection._univariate_selection import _clean_nans


class MultiSelectPercentile(SelectPercentile):

    def _custom_support_mask(self, percentile):
        check_is_fitted(self, 'scores_')

        # Cater for NaNs
        if percentile == 100:
            return np.ones(len(self.scores_), dtype=np.bool)
        elif percentile == 0:
            return np.zeros(len(self.scores_), dtype=np.bool)

        scores = _clean_nans(self.scores_)
        threshold = np.percentile(scores, 100 - percentile)
        # threshold = stats.scoreatpercentile(scores, 100 - percentile)
        mask = scores > threshold
        ties = np.where(scores == threshold)[0]
        if len(ties):
            max_feats = int(len(scores) * percentile / 100)
            kept_ties = ties[:max_feats - mask.sum()]
            mask[kept_ties] = True
        return mask

    def transform_by_percentile(self, X, percentile):
        """Reduce X to the selected features.

        Parameters
        ----------
        X : array of shape [n_samples, n_features]
            The input samples.

        percentile : int
            Percent of features to keep

        Returns
        -------
        X_r : array of shape [n_samples, n_selected_features]
            The input samples with only the selected features.
        """
        X = check_array(X, accept_sparse='csr')
        mask = self._custom_support_mask(percentile=percentile)
        if not mask.any():
            warn("No features were selected: either the data is"
                 " too noisy or the selection test too strict.",
                 UserWarning)
            return np.empty(0).reshape((X.shape[0], 0))
        if len(mask) != X.shape[1]:
            raise ValueError("X has a different shape than during fitting.")
        return X[:, safe_mask(X, mask)]



"""scikit-learn classifier wrapper ignoring some input columns."""

import re

import numpy as np
import scipy as sp
from sklearn.base import BaseEstimator, ClassifierMixin
from sklearn.utils.multiclass import unique_labels
from sklearn.utils import (
    check_array,
    check_X_y,
)


class _BaseColumnIgnoringClassifier(BaseEstimator, ClassifierMixin):
    """A base class for sklearn classifier wrappers that ignores input columns.
    """

    def __init__(self, clf):
        self.clf = clf
        if hasattr(self.clf, "decision_function"):
            setattr(self, 'decision_function', self._hidden_decision_function)
        if hasattr(self.clf, "predict_proba"):
            setattr(self, 'predict_proba', self._hidden_predict_proba)

    def fit(self, X, y, validate=True, sparse=False):
        """Fits the classifier

        Parameters
        ----------
        X : pandas.DataFrame, shape = [n_samples, n_features]
            The training input samples.
        y : array-like, shape = [n_samples]
            The target values. An array of int.
        validate : bool, default True
            If set, input arrays type is validated to be ndarray.
        sparse : bool, default False
            If set, X is assumed to be a pandas.SparseDataFrame object.

        Returns
        -------
        self : object
            Returns self.
        """
        if sparse:
            self.classes_ = unique_labels(y.to_dense())
        else:
            self.classes_ = unique_labels(y)
        # self.classes_ = np.unique(y)
        inner_X = self._transform_X(X)
        if validate:
            inner_X, y = check_X_y(inner_X, y)
        if sparse:
            inner_X = sp.sparse.csr_matrix(inner_X.to_coo())
            y = np.array(y)
        self.clf = self.clf.fit(inner_X, y)
        return self

    def predict(self, X):
        """Predict labels.

        Parameters
        ----------
        X : array-like of shape = [n_samples, n_features]
            The input samples.

        Returns
        -------
        y : array of int of shape = [n_samples]
            Predicted labels for the given inpurt samples.
        """
        inner_X = check_array(self._transform_X(X))
        return self.clf.predict(inner_X)

    def _hidden_predict_proba(self, X):
        """Predict class probabilities for X.

        Parameters
        ----------
        X : array-like of shape = [n_samples, n_features]
            The input samples.

        Returns
        -------
        p : array of shape = [n_samples, n_classes]
            The class probabilities of the input samples. The order of the
            classes corresponds to that in the attribute classes_.
        """
        inner_X = check_array(self._transform_X(X))
        return self.clf.predict_proba(inner_X)

    def _hidden_decision_function(self, X):
        """Predict confidence scores for samples.

        The confidence score for a sample is the signed distance of that sample
        to the hyperplane.

        Parameters
        ----------
        X : array-like of shape = [n_samples, n_features]
            The input samples.

        Returns
        -------
        array, shape=(n_samples,) if n_classes == 2 else (n_samples, n_classes)
            onfidence scores per (sample, class) combination. In the binary
            case, confidence score for self.classes_[1] where >0 means this
            class would be predicted.
        """
        inner_X = check_array(self._transform_X(X))
        return self.clf.decision_function(inner_X)


class IxColIgnoringClassifier(_BaseColumnIgnoringClassifier):
    """A sklearn classifier wrapper that ignores input columns by index.

    Parameters
    ----------
    clf : classifier object implementing 'fit'
        The object to use to fit the data.
    col_ignore : list
        A list of indices of columns to ignore.
    """
    def __init__(self, clf, col_ignore):
        super(IxColIgnoringClassifier, self).__init__(clf=clf)
        self.col_ignore = col_ignore

    def _transform_X(self, X):
        X = np.array(X)
        col_keep = [
            i for i in range(X.shape[1]) if i not in self.col_ignore]
        return X[:, col_keep]


class ObjColIgnoringClassifier(_BaseColumnIgnoringClassifier):
    """A sklearn classifier wrapper that ignores object columns in dataframes.

    Parameters
    ----------
    clf : classifier object implementing 'fit'
        The object to use to fit the data.
    """
    def __init__(self, clf):
        super(ObjColIgnoringClassifier, self).__init__(clf=clf)
        self.col_to_drop = None

    def _transform_X(self, X):
        if self.col_to_drop is None:
            self.col_to_drop = []
            for colname, dtype in X.dtypes.items():
                if dtype == object:
                    self.col_to_drop.append(colname)
                else:
                    try:
                        if dtype.kind == 'O':
                            self.col_to_drop.append(colname)
                    except AttributeError:
                        pass
        return X.drop(self.col_to_drop, axis=1)


class PatternColIgnoringClassifier(_BaseColumnIgnoringClassifier):
    """A sklearn classifier wrapper that ignores columns in dataframes by name.

    Parameters
    ----------
    clf : classifier object implementing 'fit'
        The object to use to fit the data.
    pattern : str
        The name pattern to match column names with.
    exclude : bool, default True
        If set to True (default), all columns matching the pattern are ignored.
        Otherwise, all columns NOT matching the pattern are ignored.
    """
    def __init__(self, clf, pattern, exclude=False):
        super(PatternColIgnoringClassifier, self).__init__(clf=clf)
        self.pattern = pattern
        self.exclude = exclude
        if self.exclude:
            self.decision_func_ = lambda x: x
        else:
            self.decision_func_ = lambda x: not x

    def _transform_X(self, X):
        col_to_drop = [
            col_name for col_name in X.columns
            if self.decision_func_(re.match(self.pattern, col_name))
        ]
        return X.drop(col_to_drop, axis=1)



"""Test colunm-ignoring classifier wrapper."""

import pytest
import pandas as pd
from sklearn.svm import SVC
from pdutil.transform import x_y_by_col_lbl

from skutil.estimators import (
    IxColIgnoringClassifier,
    ObjColIgnoringClassifier,
)


BASE_DATA = [
    [1, 'asd', 34, 0],
    [2, '98', 993, 1],
]


def test_col_ignoring_clf():
    clf = SVC(probability=True, gamma='auto')
    df = pd.DataFrame(data=BASE_DATA, columns=['x1', 'x2', 'x3', 'y'])
    X, y = x_y_by_col_lbl(df=df, y_col_lbl='y')
    with pytest.raises(ValueError):
        clf.fit(X, y)

    ignore_clf = IxColIgnoringClassifier(
        clf=clf,
        col_ignore=[list(df.columns).index('x2')],
    )
    fitted_ignore_clf = ignore_clf.fit(X, y)
    assert fitted_ignore_clf == ignore_clf

    test_data = [[1, '233', 30]]
    test_df = pd.DataFrame(test_data)
    res = fitted_ignore_clf.predict(test_df)
    assert len(res) == 1

    res2 = fitted_ignore_clf.predict_proba(test_df)
    assert len(res2) == 1
    assert len(res2[0] == 2)
    for proba in res2[0]:
        assert isinstance(proba, float)
        assert proba <= 1
        assert proba >= 0

    ignore_clf = IxColIgnoringClassifier(
        clf=clf,
        col_ignore=[list(df.columns).index('x2')],
    )
    fitted_ignore_clf = ignore_clf.fit(X, y, validate=False)
    assert fitted_ignore_clf == ignore_clf

    test_data = [[1, '233', 30]]
    test_df = pd.DataFrame(test_data)
    res = fitted_ignore_clf.predict(test_df)
    assert len(res) == 1


def test_obj_col_ignoring_clf():
    clf = SVC(probability=True, gamma='auto')
    df = pd.DataFrame(data=BASE_DATA, columns=['x1', 'x2', 'x3', 'y'])
    X, y = x_y_by_col_lbl(df=df, y_col_lbl='y')
    with pytest.raises(ValueError):
        clf.fit(X, y)

    ignore_clf = ObjColIgnoringClassifier(clf=clf)
    fitted_ignore_clf = ignore_clf.fit(X, y)
    assert fitted_ignore_clf == ignore_clf

    test_data = [[1, '233', 30]]
    test_df = pd.DataFrame(test_data, columns=['x1', 'x2', 'x3'])
    res = fitted_ignore_clf.predict(test_df)
    assert len(res) == 1

    res2 = fitted_ignore_clf.predict_proba(test_df)
    assert len(res2) == 1
    assert len(res2[0] == 2)
    for proba in res2[0]:
        assert isinstance(proba, float)
        assert proba <= 1
        assert proba >= 0



"""Test the ConstainedParameterGrid class."""

from sklearn.model_selection import ParameterGrid

from skutil.model_selection import ConstrainedParameterGrid


PARAMS1 = {
    'a': [8, 9],
    'b': [1, 2, 3],
    'c': [5, 6],
}
CONSTRAINTS1 = [{'a': [8], 'b': [3]}]


def test_base():
    grid = ParameterGrid(PARAMS1)
    cgrid = ConstrainedParameterGrid(
        param_grid=PARAMS1,
        bad_comb=CONSTRAINTS1,
    )
    reg_param_sets = list(grid)
    const_param_sets = list(cgrid)
    assert len(reg_param_sets) == 12
    assert len(const_param_sets) == 10
    for param_set in const_param_sets:
        assert not (param_set['a'] == 8 and param_set['b'] == 3)


def test_no_constraints():
    grid = ParameterGrid(PARAMS1)
    cgrid = ConstrainedParameterGrid(
        param_grid=PARAMS1,
        bad_comb=None,
    )
    reg_param_sets = list(grid)
    const_param_sets = list(cgrid)
    assert len(reg_param_sets) == 12
    assert len(const_param_sets) == 12
    found_bad = False
    for param_set in const_param_sets:
        if param_set['a'] == 8 and param_set['b'] == 3:
            found_bad = True
    assert found_bad


PARAMS2 = {
    'a': [1, 2],
    'b': ['Xor', 'Yank'],
}


def test_partial_with_str_assignment():
    cgrid = ConstrainedParameterGrid(
        param_grid=PARAMS2,
        bad_comb=None,
    )
    partial_grid = cgrid.partial({'a': 1})
    param_sets = list(cgrid)
    partial_param_sets = list(partial_grid)
    assert len(param_sets) == 4
    assert len(partial_param_sets) == 2
    found_bad = False
    for param_set in partial_param_sets:
        if param_set['a'] == 2:
            found_bad = True
    assert not found_bad
